@@ -11,6 +11,7 @@
 #' @param stroke numerical indicating width of stroke relative to the size of
 #' the text. Ignored if less than zero.
 #' @param stroke.color any valid colour.
+#' @param label.placement A function for placing labels (see `label_placement_flattest()`).
 #'
 #' @details
 #' Is best used with a previous call to [ggplot2::stat_contour] with the same
@@ -27,6 +28,21 @@
 #'
 #' g + geom_text_contour(aes(z = value), stroke = 0.2)
 #'
+#' g + geom_text_contour(aes(z = value), rotate = FALSE)
+#'
+#' g + geom_text_contour(aes(z = value),
+#'                       label.placement = label_placement_random())
+#'
+#' g + geom_text_contour(aes(z = value),
+#'                       label.placement = label_placement_n(3))
+#'
+#'
+#' g + geom_text_contour(aes(z = value),
+#'                       label.placement = label_placement_flattest())
+#'
+#' g + geom_text_contour(aes(z = value),
+#'                       label.placement = label_placement_flattest(ref_angle = 90))
+#'
 #' @section Aesthetics:
 #' \code{geom_text_contour} understands the following aesthetics (required aesthetics are in bold):
 #'
@@ -48,15 +64,16 @@
 #'
 #'
 #' @export
-#' @import ggplot2 data.table
 #' @family ggplot2 helpers
+#' @importFrom ggplot2 .pt
 geom_text_contour <- function(mapping = NULL, data = NULL,
                       stat = "text_contour",
                       position = "identity",
                       ...,
                       min.size = 5,
-                      skip = 0,
+                      skip = 1,
                       rotate = TRUE,
+                      label.placement = label_placement_flattest(),
                       parse = FALSE,
                       nudge_x = 0,
                       nudge_y = 0,
@@ -96,6 +113,7 @@ geom_text_contour <- function(mapping = NULL, data = NULL,
             stroke.color = stroke.color,
             # xwrap = xwrap,
             # ywrap = ywrap,
+            label.placement = label.placement,
             na.rm = na.rm,
             ...
         )
@@ -116,16 +134,28 @@ GeomTextContour <- ggplot2::ggproto("GeomTextContour", ggplot2::Geom,
    draw_panel = function(data, panel_params, coord, parse = FALSE,
                          na.rm = FALSE, check_overlap = FALSE, min.size = 20,
                          skip = 1, rotate = FALSE, gap = NULL,
+                         label.placement = label_placement_flattest(),
                          stroke = 0, stroke.colour = "white") {
        data <- data.table::as.data.table(coord$transform(data, panel_params))
        min.size <- ceiling(min.size)
        if (min.size %% 2 == 0) {
            min.size <- min.size - 1
        }
-       # Get points of labels
-       data <- .label.position(data.table::copy(data), min.size, skip, rotate)
+
+       breaks <- unique(data$level)
+       breaks.cut <- breaks[seq(1, length(breaks), by = skip + 1)]
+
+       # browser()
+       data <- data[level %in% breaks.cut]
+
+       data <- data[, unique(.SD, by = c("x", "y")),by = .(group, piece)]
+
+       data[, N := .N, by = .(group, piece)]
+       data <- data[N > 3]
+       data <- data[N >= min.size][, N := NULL]
+
        if (nrow(data) == 0) {
-           return(nullGrob())
+           return(grid::nullGrob())
        }
 
        ## Original ggplot2 here.
@@ -135,26 +165,33 @@ GeomTextContour <- ggplot2::ggproto("GeomTextContour", ggplot2::Geom,
        }
 
        if (is.character(data$vjust)) {
-           data$vjust <- ggplot2:::compute_just(data$vjust, data$y)
+           vjust <- ggplot2:::compute_just(data$vjust, data$y)[1]
+       } else {
+           vjust <- data$vjust[1]
        }
        if (is.character(data$hjust)) {
-           data$hjust <- ggplot2:::compute_just(data$hjust, data$x)
+           hjust <- ggplot2:::compute_just(data$hjust, data$x)[1]
+       } else {
+           hjust <- data$hjust[1]
        }
 
-       shadowtext(
+       contourTextGrob(
            lab,
-           data$x, data$y, default.units = "native",
-           hjust = data$hjust, vjust = data$vjust,
-           dx = data$dx, dy = data$dy,
+           data$x, data$y,
+           group = interaction(data$group, data$piece),
+           default.units = "native",
+           hjust = hjust, vjust = vjust,
            bg.r = stroke, bg.color = stroke.colour,
-           gp = grid::gpar(
+           position = label.placement,
+
                col = scales::alpha(data$colour, data$alpha),
                fontsize = data$size * .pt,
                fontfamily = data$family,
                fontface = data$fontface,
-               lineheight = data$lineheight
-           ),
-           check.overlap = check_overlap
+               lineheight = data$lineheight,
+
+           check.overlap = check_overlap,
+           rotate = rotate
        )
 
    },
@@ -163,97 +200,3 @@ GeomTextContour <- ggplot2::ggproto("GeomTextContour", ggplot2::Geom,
 )
 
 
-.cont.angle <- function(x, y, gap) {
-    N <- length(x)
-    gap <- ceiling((gap-1)/2)
-    fill <- rep(NA, gap/2)
-    dx <- c(fill, diff(x, gap), fill)
-    dy <- c(fill, diff(y, gap), fill)
-    angle <- atan2(dy, dx)*180/pi
-    angle <- ifelse(angle > 180, angle - 180, angle)
-    angle <- ifelse(angle > 90, angle - 180, angle)
-    angle <- ifelse(angle < -90, angle + 180, angle)
-    angle
-}
-
-.label.position <- function(data, min.size, skip, rotate) {
-    data <- as.data.table(data)
-
-    breaks <- unique(data$level)
-    breaks.cut <- breaks[seq(1, length(breaks), by = skip + 1)]
-    data <- data[level %in% breaks.cut]
-    data[, id := seq_len(.N), by = piece]
-    data[, c("dx", "dy") := .(.derv(x, id), .derv(y, id)),
-         by = .(piece)]
-
-    # Safety strip around the edges (10%)
-    safe <- c(0, 1) + 0.1*c(+1, -1)
-    data <- data[x %between% safe &
-                     y %between% safe]
-
-    data[, N := .N, by = piece]
-    data <- data[N >= min.size]
-    if (nrow(data) == 0) {
-        return(data)
-    }
-    # if (rotate == TRUE) {
-        data[, angle := .cont.angle(x, y, min.size), by = piece]
-    # }
-
-    # Check if point has minimum variance
-    # data[, var := minvar(x, y), by = .(piece)]
-    # data[, curvature := .curvature(x, y), by = piece]
-    # data <- data[data[, .I[which.min(abs(curvature))], by = piece]$V1]
-    # data[is.na(var), var := FALSE]
-    # data <- data[var == TRUE][, head(.SD, 1), by = piece]
-
-    data[, min := straight(x, y, min.size), by = piece]
-    # data <- data[min == TRUE]
-    # data[, min := angle == min(angle, na.rm = T), by = piece]
-    # data[, min := atan2(dy, dx) == min(atan2(dy, dx), na.rm = TRUE), by = piece]
-    data <- data[!is.na(dx + dy)]
-    if (rotate == FALSE) data[, c("dx", "dy") := .(0, 0)]
-
-    return(data[min == TRUE, head(.SD, 1), by = piece])
-}
-
-straight <- function(x, y, gap) {
-    N <- length(x)
-    gap <- ceiling((gap-1)/2)
-    if (N < gap) {
-        gap <- N
-    }
-    if (is_closed(x, y)) {
-        x <- c(x[(N-gap):(N-1)], x, x[2:(gap+1)])
-        y <- c(y[(N-gap):(N-1)], y, y[2:(gap+1)])
-        var <- vapply((gap+1):(N+gap), function(i) {
-            dx <- diff(x[(i-gap):(i+gap)])
-            dy <- diff(y[(i-gap):(i+gap)])
-            a <- dy/dx
-            var(a)
-        }, FUN.VALUE = 1)
-    } else {
-        var <- c(rep(NA, gap), vapply((gap+1):(N-gap), function(i) {
-            dx <- diff(x[(i-gap):(i+gap)])
-            dy <- diff(y[(i-gap):(i+gap)])
-            a <- dy/dx
-            var(a)
-        }, FUN.VALUE = 1), rep(NA, gap))
-    }
-    return(var == min(var, na.rm = TRUE))
-}
-
-# from https://stackoverflow.com/questions/21868353/drawing-labels-on-flat-section-of-contour-lines-in-ggplot2
-minvar <- function (x, y){
-    if (length(x) < 4) return(rep(FALSE, length(x)))
-    N <- length(x)
-    xdiffs <- c(NA, diff(x)) #c(NA, x[3:N] - x[1:(N-2)], NA)
-    ydiffs <- c(NA, diff(y)) #c(NA, y[3:N] - y[1:(N-2)], NA)
-    avgGradient <- ydiffs/xdiffs
-    variance <- abs(avgGradient)
-    # squareSum <- avgGradient * avgGradient
-    # variance <- (squareSum - (avgGradient * avgGradient) / N) / N
-    # change!! this causes problems if length(variance) < 4
-    variance <- c(NA, variance[2:(N-1)], NA)
-    return(variance == min(variance, na.rm = TRUE))
-}
