@@ -1,6 +1,5 @@
 #' @rdname geom_contour_fill
 #' @export
-#' @import ggplot2
 stat_contour_fill <- function(mapping = NULL, data = NULL,
                               geom = "polygon", position = "identity",
                               ...,
@@ -8,10 +7,8 @@ stat_contour_fill <- function(mapping = NULL, data = NULL,
                               bins = NULL,
                               binwidth = NULL,
                               global.breaks = TRUE,
-                              # na.rm = FALSE,
+                              kriging = FALSE,
                               na.fill = FALSE,
-                              # xwrap = NULL,
-                              # ywrap = NULL,
                               show.legend = NA,
                               inherit.aes = TRUE) {
     .check_wrap_param(list(...))
@@ -30,21 +27,19 @@ stat_contour_fill <- function(mapping = NULL, data = NULL,
             bins = bins,
             binwidth = binwidth,
             global.breaks = global.breaks,
-            # xwrap = xwrap,
-            # ywrap = ywrap,
+            kriging = kriging,
             ...
         )
     )
 }
 
-#' @import ggplot2
 #' @rdname geom_contour_fill
 #' @usage NULL
 #' @format NULL
 #' @export
 StatContourFill <- ggplot2::ggproto("StatContourFill", ggplot2::Stat,
     required_aes = c("x", "y", "z"),
-    default_aes = ggplot2::aes(fill = ..int.level..),
+    default_aes = ggplot2::aes(fill = ..level_mid.., order = ..level..),
     setup_params = function(data, params) {
         if (is.null(params$global) || isTRUE(params$global.breaks)) {
             params$breaks <- setup_breaks(data,
@@ -78,7 +73,8 @@ StatContourFill <- ggplot2::ggproto("StatContourFill", ggplot2::Stat,
     compute_group = function(data, scales, bins = NULL, binwidth = NULL,
                              breaks = scales::fullseq, complete = TRUE,
                              na.rm = FALSE, xwrap = NULL,
-                             ywrap = NULL, na.fill = FALSE, global.breaks = TRUE) {
+                             ywrap = NULL, na.fill = FALSE, global.breaks = TRUE,
+                             proj = NULL, kriging = FALSE) {
         data.table::setDT(data)
 
         if (isFALSE(global.breaks)) {
@@ -90,29 +86,36 @@ StatContourFill <- ggplot2::ggproto("StatContourFill", ggplot2::Stat,
 
         data <- data[!(is.na(y) | is.na(x)), ]
 
-        if (isFALSE(na.fill)) {
+        if (!isFALSE(na.fill)) {
+            # Check if is a complete grid
+            complete.grid <- with(data, .is.regular_grid(x, y))
+            if (complete.grid == FALSE) {
+                if (complete == FALSE) {
+                    warning("data must be a complete regular grid", call. = FALSE)
+                    return(data.frame())
+                } else {
+                    # data <- data.table::setDT(tidyr::complete(data, x, y, fill = list(z = NA)))
+                    data <- .complete(data, x, y)
+                }
+            }
+
+            data <- .impute_data(data, na.fill)
+        } else {
             data <- data[!is.na(z), ]
         }
 
-        # Check if is a complete grid
-        complete.grid <- with(data, .is.regular_grid(x, y))
+        if (kriging) {
+            check_packages("kriging", "kriging")
 
-        if (complete.grid == FALSE) {
-            if (complete == FALSE) {
-                warning("data must be a complete regular grid", call. = FALSE)
+            pixels <- 40
+            data <- try(with(data, setNames(kriging::kriging(x, y, z, pixels = pixels)$map,
+                                            c("x", "y", "z"))), silent = TRUE)
+            if (inherits(data, "try-error")) {
+                warning("kriging failed. Perhaps the number of points is too small.")
                 return(data.frame())
-            } else {
-                # data <- data.table::setDT(tidyr::complete(data, x, y, fill = list(z = NA)))
-                data <- .complete(data, x, y)
             }
-        }
 
-        data <- .impute_data(data, na.fill)
-
-        nas <- nrow(data[is.na(z)])
-        if (nas != 0) {
-            warning("data must not have missing values. Use na.fill = TRUE or impute them before plotting.", call. = FALSE)
-            return(data.frame())
+            data.table::setDT(data)
         }
 
         if (!is.null(xwrap)) {
@@ -122,76 +125,82 @@ StatContourFill <- ggplot2::ggproto("StatContourFill", ggplot2::Stat,
             data <- suppressWarnings(WrapCircular(data, "y", ywrap))
         }
 
-        # jitter <- diff(range(data$z))*0.000000
-        # data$z <- data$z + rnorm(nrow(data))*jitter
-        data.table::setDF(data)
-        mean.z <- data$z[data$z %~% mean(data$z)][1]
-
-        # mean.z <- min(breaks) - 100*ggplot2::resolution(breaks, zero = FALSE)
-        range.data <- as.data.frame(vapply(data[c("x", "y", "z")], range, c(1, 2)))
-
-        # Expand data by 1 unit in all directions.
-        data <- .expand_data(data, mean.z)
 
         # Make contours
-        cont <- data.table::setDT(.contour_lines(data, breaks, complete = complete))
+        cont <- data.table::setDT(.contour_bands(data, breaks, complete = complete))
 
-        if (length(cont) == 0) {
-            warning("Not possible to generate contour data", call. = FALSE)
-            return(data.frame())
+        cont[, int.level := (level_high + level_low)/2]
+        cont[, level_mid := int.level]
+        cont[, nlevel := level_high/max(level_high)]
+
+        if (!is.null(proj)) {
+            if (is.function(proj)) {
+                cont <- proj(cont)
+            } else {
+                if (is.character(proj)) {
+                    if (!requireNamespace("proj4", quietly = TRUE)) {
+                        stop("Projection requires the proj4 package. Install it with `install.packages(\"proj4\")`")
+                    }
+                    cont <- data.table::copy(cont)[, c("x", "y") := proj4::project(list(x, y), proj,
+                                                                                           inverse = TRUE)][]
+
+                }
+            }
         }
 
-        # Ugly hack for joining disjointed contours
-        cont <- .join_contours.m(cont)
-
-        # Calculate inner fill
-        cont <- .inner_fill.m(cont, data, breaks)
-
-        # Add bounding contour, if necessary.
-        if (data.table::between(mean.z, min(breaks), max(breaks))) {
-            mean.level <- breaks[breaks %~% mean.z]
-
-            i <-  which(breaks == mean.level)
-            correction <- sign(mean.z - mean.level)
-            if (correction == 0) correction <- 1
-            correction <- (breaks[i + correction] - mean.level)/2
-
-            # Adds bounding contour
-            data.table::setDT(data)
-            Nx <- data.table::uniqueN(data$x)
-            Ny <- data.table::uniqueN(data$y)
-
-            x <- c(rep(range.data$x[1], Ny),
-                   sort(data[y == range.data$y[2], x]),
-                   rep(range.data$x[2], Ny),
-                   sort(data[y == range.data$y[1], x], decreasing = TRUE))
-            y <- c(sort(data[x == range.data$x[1], y]),
-                   rep(range.data$y[2], Nx),
-                   sort(data[x == range.data$x[2], y], decreasing = TRUE),
-                   rep(range.data$y[1], Nx))
-
-            mean.cont  <- data.frame(
-                level = mean.level,
-                x = x,
-                y = y,
-                piece = max(cont$piece) + 1,
-                int.level = mean.level + correction)
-            mean.cont$group <- factor(paste("", sprintf("%03d", mean.cont$piece), sep = "-"))
-            cont <- rbind(cont, mean.cont)
-        }
-
-        # Move contours to range of original data
-        cont$x[cont$x > range.data$x[2]] <- range.data$x[2]
-        cont$x[cont$x < range.data$x[1]] <- range.data$x[1]
-        cont$y[cont$y < range.data$y[1]] <- range.data$y[1]
-        cont$y[cont$y > range.data$y[2]] <- range.data$y[2]
-
-        # Order pieces according to area.
-        cont <- .order_fill(cont)
         cont
+
         }
 )
 
+
+
+.contour_bands <- function(data, breaks, complete = FALSE) {
+    band <- level_high <- level_low <- NULL
+
+    # From ggplot2
+    x_pos <- as.integer(factor(data$x, levels = sort(unique(data$x))))
+    y_pos <- as.integer(factor(data$y, levels = sort(unique(data$y))))
+
+    nrow <- max(y_pos)
+    ncol <- max(x_pos)
+
+    z <- matrix(NA_real_, nrow = nrow, ncol = ncol)
+    z[cbind(y_pos, x_pos)] <- data$z
+
+
+
+    cl <- isoband::isobands(x = sort(unique(data$x)),
+                            y = sort(unique(data$y)),
+                            z = z,
+                            levels_low = breaks[-length(breaks)],
+                            levels_high = breaks[-1])
+
+
+    if (length(cl) == 0) {
+        warning("Not possible to generate contour data", call. = FALSE)
+        return(data.frame())
+    }
+
+    # Convert list of lists into single data frame
+
+    cont <- data.table::rbindlist(lapply(cl, data.table::as.data.table), idcol = "band")
+
+    cont[, band := ordered(band, names(cl))]
+    cont[, c("level_low", "level_high") := data.table::tstrsplit(band, ":")]
+    cont[, `:=`(level_low = as.numeric(level_low), level_high = as.numeric(level_high))]
+    # cont[, level := paste0("(", format(as.numeric(level_low), digits = 3, trim = TRUE),
+    #                        ", ", format(as.numeric(level_high), digits = 3, trim = TRUE))]
+    # cont[, level := factor(level, ordered = TRUE)]
+
+    cont[, piece := as.numeric(interaction(band))]
+    cont[, group := factor(paste(data$group[1], sprintf("%03d", piece), sep = "-"))]
+
+
+    cont[, .(level = band, level_low, level_high, x, y, piece, group, subgroup = id)]
+
+}
+
 .order_fill <- function(cont) {
     areas <- cont[, .(area = abs(area(x, y))), by = .(piece)][
         , rank := data.table::frank(-area, ties.method = "first")]
@@ -233,8 +242,6 @@ area <- function(x, y) {
     sum(xdiff * ysum)/2
 }
 
-
-#' @import data.table
 .inner_fill <- function(cont, data, breaks) {
     # Add one more to breaks extremes
     m.level <- -breaks[2] + 2*breaks[1]
@@ -359,6 +366,7 @@ close_path <- function(x, y, range_x, range_y) {
     contours[, close := NULL]
     if (nrow(contours2) == 0) return(contours)
     contours2[, close := NULL]
+    first <- NULL
     contours2[, first := (1 == 1:.N), by = piece]
     pieces <- unique(contours2[, piece])
 
